@@ -1,5 +1,6 @@
 __author__  = 'Jonny Saunders <j@nny.fyi>'
-__version__ = '0.4.3'
+__version__ = '0.4.4.post0'
 
 from autopilot.setup import setup_autopilot
-from autopilot.utils.registry import get, get_task, get_hardware, get_names
\ No newline at end of file
+from autopilot.utils.registry import get, get_task, get_hardware, get_names
+from autopilot.utils.hydration import dehydrate, hydrate
\ No newline at end of file
