@@ -6,7 +6,6 @@ Currently named subject, but will likely be refactored to include other data
 models should the need arise.
 
 """
-
 # TODO: store pilot in biography
 import os
 import sys
@@ -20,14 +19,13 @@ import warnings
 import typing
 import warnings
 from copy import copy
-from autopilot.tasks import GRAD_LIST, TASK_LIST
+import autopilot
 from autopilot import prefs
 from autopilot.stim.sound.sounds import STRING_PARAMS
+from autopilot.core.loggers import init_logger
+
+import queue
 
-if sys.version_info >= (3,0):
-    import queue
-else:
-    import Queue as queue
 
 # suppress pytables natural name warnings
 warnings.simplefilter('ignore', category=tables.NaturalNameWarning)
@@ -95,6 +93,10 @@ class Subject(object):
             new (bool): if True, a new file is made (a new file is made if one does not exist anyway)
             biography (dict): If making a new subject file, a dictionary with biographical data can be passed
         """
+        # try to get name first off for logger
+        self.name = name
+        self.logger = init_logger(self)
+
         self.STRUCTURE = [
             ('/data', '/', 'data', 'group'),
             ('/history', '/', 'history' 'group'),
@@ -118,17 +120,19 @@ class Subject(object):
 
         if not name:
             if not file:
-                Exception('Need to either have a name or a file, how else would we find the .h5 file?')
+                raise Exception('Need to either have a name or a file, how else would we find the .h5 file?')
             if not os.path.isfile(file):
-                Exception('no file was found at passed file: {}'.format(file))
+                raise Exception('no file was found at passed file: {}'.format(file))
             self.file = file
         else:
             if file:
-                Warning('file passed, but so was name, defaulting to using name + dir')
+                self.logger.warning('file passed, but so was name, defaulting to using name + dir')
 
             self.name = str(name)
             self.file = os.path.join(dir, name + '.h5')
             if new or not os.path.isfile(self.file):
+                # set new to true in case new'd from absence of file
+                new = True
                 self.new_subject_file(biography)
 
         # before we open, make sure we have the stuff we need
@@ -140,7 +144,7 @@ class Subject(object):
             try:
                 self.name = h5f.root.info._v_attrs['name']
             except KeyError:
-                Warning('No Name attribute saved, trying to recover from filename')
+                self.logger.warning('No Name attribute saved, trying to recover from filename')
                 self.name = os.path.splitext(os.path.split(file)[-1])[0]
 
 
@@ -157,6 +161,9 @@ class Subject(object):
             self.current = json.loads(protocol_string)
             self.step = int(current_node.attrs['step'])
             self.protocol_name = current_node.attrs['protocol_name']
+        elif not new:
+            # if we're not being created for the first time, warn that there is no protocol assigned to the subject
+            self.logger.warning('Subject has no protocol assigned!')
 
         # get last session number if we have it
         try:
@@ -181,12 +188,13 @@ class Subject(object):
         history_row['time'] = self.get_timestamp()
         try:
             history_row['hash'] = prefs.get('HASH')
+            # FIXME: less implicit way of getting hash plz
         except AttributeError:
             history_row['hash'] = ''
         history_row.append()
 
         # we have to always open and close the h5f
-        _ = self.close_hdf(h5f)
+        self.close_hdf(h5f)
 
     def open_hdf(self, mode='r+'):
         """
@@ -226,7 +234,7 @@ class Subject(object):
         """
         with self.lock:
             h5f.flush()
-            return h5f.close()
+            h5f.close()
 
     def new_subject_file(self, biography):
         """
@@ -291,23 +299,12 @@ class Subject(object):
             try:
                 node = h5f.get_node(node[0])
             except tables.exceptions.NoSuchNodeError:
-                #pdb.set_trace()
                 # try to make it
-                # python 3 compatibility
-                if sys.version_info >= (3,0):
-                    if isinstance(node[3], str):
-                        if node[3] == 'group':
-                            h5f.create_group(node[1], node[2])
-                    elif issubclass(node[3], tables.IsDescription):
-                        h5f.create_table(node[1], node[2], description=node[3])
-
-                # python 2
-                else:
-                    if isinstance(node[3], str):
-                        if node[3] == 'group':
-                            h5f.create_group(node[1], node[2])
-                    elif issubclass(node[3], tables.IsDescription):
-                        h5f.create_table(node[1], node[2], description=node[3])
+                if isinstance(node[3], str):
+                    if node[3] == 'group':
+                        h5f.create_group(node[1], node[2])
+                elif issubclass(node[3], tables.IsDescription):
+                    h5f.create_table(node[1], node[2], description=node[3])
 
         self.close_hdf(h5f)
 
@@ -348,6 +345,8 @@ class Subject(object):
             step (int): When type is 'param', changes the parameter at a particular step,
                 otherwise the current step is used.
         """
+        self.logger.info(f'Updating subject {self.name} history - type: {type}, name: {name}, value: {value}, step: {step}')
+
         # Make sure the updates are written to the subject file
         if type == 'param':
             if not step:
@@ -424,7 +423,7 @@ class Subject(object):
         if not os.path.exists(protocol):
             fullpath = os.path.join(prefs.get('PROTOCOLDIR'), protocol)
             if not os.path.exists(fullpath):
-                Exception('Could not find either {} or {}'.format(protocol, fullpath))
+                raise Exception('Could not find either {} or {}'.format(protocol, fullpath))
             protocol = fullpath
 
         # Set name and step
@@ -445,7 +444,6 @@ class Subject(object):
         # Check if there is an existing protocol, archive it if there is.
         if "/current" in h5f:
             _ = self.close_hdf(h5f)
-            self.update_history(type='protocol', name=protocol_name, value = prot_dict)
             self.stash_current()
             h5f = self.open_hdf()
 
@@ -483,7 +481,7 @@ class Subject(object):
         # memory, we can just keep appending to keep things simple.
         for i, step in enumerate(self.current):
             # First we get the task class for this step
-            task_class = TASK_LIST[step['task_type']]
+            task_class = autopilot.get_task(step['task_type'])
             step_name = step['step_name']
             # group name is S##_'step_name'
             group_name = "S{:02d}_{}".format(i, step_name)
@@ -534,6 +532,7 @@ class Subject(object):
 
                     h5f.create_table(step_group, "trial_data", trial_descriptor)
                 else:
+                    self.logger.warning('No trial data descriptor found, making default table with session and trial_num')
                     h5f.create_table(step_group, "trial_data", {'session': tables.Int32Col(), 'trial_num': tables.Int32Col()})
             except tables.NodeError:
                 # we already have made this table, that's fine
@@ -559,7 +558,10 @@ class Subject(object):
         _ = self.close_hdf(h5f)
 
         # Update history
-        self.update_history('protocol', protocol_name, self.current)
+        self.update_history(type='protocol', name=protocol_name, value=self.current)
+        self.update_history(type='step',
+                            name=self.current[self.step]['step_name'],
+                            value=self.step)
 
     def flush_current(self):
         """
@@ -569,6 +571,7 @@ class Subject(object):
         Used to make sure the stored .json representation of the current task stays up to date
         with the params set in the subject object
         """
+
         h5f = self.open_hdf()
         h5f.remove_node('/current')
         current_node = filenode.new_node(h5f, where='/', name='current')
@@ -576,6 +579,7 @@ class Subject(object):
         current_node.attrs['step'] = self.step
         current_node.attrs['protocol_name'] = self.protocol_name
         self.close_hdf(h5f)
+        self.logger.debug('current protocol flushed')
 
     def stash_current(self):
         """
@@ -602,6 +606,7 @@ class Subject(object):
 
         h5f.remove_node('/current')
         self.close_hdf(h5f)
+        self.logger.debug('current protocol stashed')
 
     def prepare_run(self):
         """
@@ -615,10 +620,22 @@ class Subject(object):
             Dict: the parameters for the current step, with subject id, step number,
                 current trial, and session number included.
         """
+        if self.current is None:
+            e = RuntimeError('No task assigned to subject, cant prepare_run. use Subject.assign_protocol or protocol reassignment wizard in the terminal GUI')
+            self.logger.exception(f"{e}")
+            raise e
+
 
         trial_table = None
         cont_table = None
 
+        # get step history
+        try:
+            step_df = self.get_step_history(use_history=True)
+        except Exception as e:
+            self.logger.exception(f"Couldnt get step history to trim data given to graduation objects, got exception {e}")
+            step_df = None
+
         h5f = self.open_hdf()
 
         # Get current task parameters and handles to tables
@@ -629,83 +646,118 @@ class Subject(object):
         group_name = "/data/{}/S{:02d}_{}".format(self.protocol_name, self.step, step_name)
         #try:
 
+        # tasks without TrialData will have some default table, so this should always be present
         trial_table = h5f.get_node(group_name, 'trial_data')
-        #self.trial_row = self.trial_table.row
-        #self.trial_keys = self.trial_table.colnames
+
+        ##################################3
+        # first try and find some timestamp column to filter past data we give to the graduation object
+        # in case the subject has been stepped back down to a previous stage, for example
+        # FIXME: Hardcoding parameter names, should have a guaranteed 'trial_timestamp' column for each trial
+        slice_start = 0
+        try:
+            ts_cols = [col for col in trial_table.colnames if 'timestamp' in col]
+            # just use the first timestamp column
+            if len(ts_cols) > 0:
+                trial_ts = pd.DataFrame({'timestamp': trial_table.col(ts_cols[0])})
+                trial_ts['timestamp'] = pd.to_datetime(trial_ts['timestamp'].str.decode('utf-8'))
+            else:
+                self.logger.warning(
+                    'No timestamp column could be found in trial data, cannot trim data given to graduation objects')
+                trial_ts = None
+
+            if trial_ts is not None and step_df is not None:
+                # see where, if any, the timestamp column is older than the last time the step was changed
+                good_rows = np.where(trial_ts['timestamp'] >= step_df['timestamp'].iloc[-1])[0]
+                if len(good_rows) > 0:
+                    slice_start = np.min(good_rows)
+                # otherwise if it's because we found no good rows but have trials,
+                # we will say not to use them, otherwise we say not to use them by
+                # slicing at the end of the table
+                else:
+                    slice_start = trial_table.nrows
+
+        except Exception as e:
+            self.logger.exception(
+                f"Couldnt trim data given to graduation objects with step change history, got exception {e}")
+
+        trial_tab = trial_table.read(start=slice_start)
+        trial_tab_keys = tuple(trial_tab.dtype.fields.keys())
+
+        ##############################
 
         # get last trial number and session
         try:
-            self.current_trial = trial_table.cols.trial_num[-1]+1
+            self.current_trial = trial_tab['trial_num'][-1]+1
         except IndexError:
+            if 'trial_num' not in trial_tab_keys:
+                self.logger.info('No previous trials detected, setting current_trial to 0')
             self.current_trial = 0
 
         # should have gotten session from current node when we started
-
+        # so sessions increment over the lifespan of the subject, even if
+        # reassigned.
         if not self.session:
             try:
-                self.session = trial_table.cols.session[-1]
+                self.session = trial_tab['session'][-1]
             except IndexError:
+                if 'session' not in trial_tab_keys:
+                    self.logger.warning('previous session couldnt be found, setting to 0')
                 self.session = 0
 
         self.session += 1
         h5f.root.info._v_attrs['session'] = self.session
         h5f.flush()
 
-        # try:
-        #     self.session = trial_table.cols.session[-1]+1
-        # except IndexError:
-        #     self.session = 0
-
         # prepare continuous data group and tables
-        task_class = TASK_LIST[task_params['task_type']]
+        task_class = autopilot.get_task(task_params['task_type'])
         cont_group = None
         if hasattr(task_class, 'ContinuousData'):
-
             cont_group = h5f.get_node(group_name, 'continuous_data')
             try:
                 session_group = h5f.create_group(cont_group, "session_{}".format(self.session))
             except tables.NodeError:
                 session_group = h5f.get_node(cont_group, "session_{}".format(self.session))
-
             # don't create arrays for each dtype here, we will create them as we receive data
 
-        # try:
-        #     #cont_table = h5f.get_node(group_name, 'continuous_data')
-        #     #self.cont_row   = self.cont_table.row
-        #     #self.cont_keys  = self.cont_table.colnames
-        # except:
-        #     pass
+        # if (trial_table is None) and (cont_group is None):
+        #     raise Exception("No data tables exist for step {}! Is there a Trial or Continuous data descriptor in the task class?".format(self.step))
 
-        if not any([cont_group, trial_table]):
-            Exception("No data tables exist for step {}! Is there a Trial or Continuous data descriptor in the task class?".format(self.step))
         # TODO: Spawn graduation checking object!
+        self.graduation = None
         if 'graduation' in task_params.keys():
-            grad_type = task_params['graduation']['type']
-            grad_params = task_params['graduation']['value'].copy()
+            try:
+                grad_type = task_params['graduation']['type']
+                grad_params = task_params['graduation']['value'].copy()
 
-            # add other params asked for by the task class
-            grad_obj = GRAD_LIST[grad_type]
+                # add other params asked for by the task class
+                grad_obj = autopilot.get('graduation', grad_type)
 
-            if grad_obj.PARAMS:
-                # these are params that should be set in the protocol settings
-                for param in grad_obj.PARAMS:
-                    #if param not in grad_params.keys():
-                    # for now, try to find it in our attributes
-                    # TODO: See where else we would want to get these from
-                    if hasattr(self, param):
-                        grad_params.update({param:getattr(self, param)})
+                if grad_obj.PARAMS:
+                    # these are params that should be set in the protocol settings
+                    for param in grad_obj.PARAMS:
+                        #if param not in grad_params.keys():
+                        # for now, try to find it in our attributes
+                        # but don't overwrite if it already has what it needs in case
+                        # of name overlap
+                        # TODO: See where else we would want to get these from
+                        if hasattr(self, param) and param not in grad_params.keys():
+                            grad_params.update({param:getattr(self, param)})
 
-            if grad_obj.COLS:
-                # these are columns in our trial table
-                for col in grad_obj.COLS:
-                    try:
-                        grad_params.update({col: trial_table.col(col)})
-                    except KeyError:
-                        Warning('Graduation object requested column {}, but it was not found in the trial table'.format(col))
+                if grad_obj.COLS:
+                    # these are columns in our trial table
 
-            #grad_params['value']['current_trial'] = str(self.current_trial) # str so it's json serializable
-            self.graduation = grad_obj(**grad_params)
-            self.did_graduate.clear()
+                    # then give the data to the graduation object
+                    for col in grad_obj.COLS:
+                        try:
+                            grad_params.update({col: trial_tab[col]})
+                        except KeyError:
+                            self.logger.warning('Graduation object requested column {}, but it was not found in the trial table'.format(col))
+
+                #grad_params['value']['current_trial'] = str(self.current_trial) # str so it's json serializable
+                self.graduation = grad_obj(**grad_params)
+                self.did_graduate.clear()
+            except Exception as e:
+                self.logger.exception(f'Exception in graduation parameter specification, graduation is disabled.\ngot error: {e}')
         else:
             self.graduation = None
 
@@ -749,7 +801,7 @@ class Subject(object):
         step_name = task_params['step_name']
 
         # file structure is '/data/protocol_name/##_step_name/tables'
-        group_name = "/data/{}/S{:02d}_{}".format(self.protocol_name, self.step, step_name)
+        group_name = f"/data/{self.protocol_name}/S{self.step:02d}_{step_name}"
         #try:
         trial_table = h5f.get_node(group_name, 'trial_data')
         trial_keys = trial_table.colnames
@@ -764,11 +816,8 @@ class Subject(object):
             session_group = h5f.get_node(continuous_group, 'session_{}'.format(self.session))
             cont_data = continuous_group._v_attrs['data']
 
-
             cont_tables = {}
             cont_rows = {}
-            #continuous_keys  = continuous_table.colnames
-            #continous_row    = continuous_table.row
         except AttributeError:
             continuous_table = False
 
@@ -803,7 +852,7 @@ class Subject(object):
                                                                        temp_timestamp_arr.shape)
 
                             except KeyError:
-                                Warning('no timestamp sent with continuous data')
+                                self.logger.warning('no timestamp sent with continuous data')
                                 continue
 
 
@@ -855,7 +904,7 @@ class Subject(object):
                         else:
                             # we have more than one row with this trial_num.
                             # shouldn't happen, but we dont' want to throw any data away
-                            Warning('Found multiple rows with same trial_num: {}'.format(data['trial_num']))
+                            self.logger.warning('Found multiple rows with same trial_num: {}'.format(data['trial_num']))
                             # continue just for data conservancy's sake
                             trial_row.append()
 
@@ -867,25 +916,24 @@ class Subject(object):
                             trial_row[k] = v
                         except KeyError:
                             # TODO: Logging here
-                            Warning("Data dropped: key: {}, value: {}".format(k, v))
+                            self.logger.warning("Data dropped: key: {}, value: {}".format(k, v))
 
                 # TODO: Or if all the values have been filled, shouldn't need explicit TRIAL_END flags
                 if 'TRIAL_END' in data.keys():
                     trial_row['session'] = self.session
-                    trial_row.append()
-                    trial_table.flush()
                     if self.graduation:
                         # set our graduation flag, the terminal will get the rest rolling
                         did_graduate = self.graduation.update(trial_row)
                         if did_graduate is True:
                             self.did_graduate.set()
+                    trial_row.append()
+                    trial_table.flush()
 
                 # always flush so that our row iteration routines above will find what they're looking for
                 trial_table.flush()
             except Exception as e:
-                # TODO: Get logger and log this
                 # we shouldn't throw any exception in this thread, just log it and move on
-                print(e)
+                self.logger.exception(f'exception in data thread: {e}')
 
         self.close_hdf(h5f)
 
@@ -907,7 +955,7 @@ class Subject(object):
         self.thread.join(5)
         self.running = False
         if self.thread.is_alive():
-            Warning('Data thread did not exit')
+            self.logger.warning('Data thread did not exit')
 
     def to_csv(self, path, task='current', step='all'):
         """
@@ -1068,15 +1116,27 @@ N Sessions: {}""".format(self.name, path, df.shape[0], len(df.session.unique()))
         h5f = self.open_hdf()
         if use_history:
             history = h5f.root.history.history
-            # return a dataframe of step number, datetime and step name
-            step_df = pd.DataFrame([(x['value'], x['time'], x['name']) for x in history.iterrows() if x['type'] == 'step'])
+            step_df = pd.DataFrame(history.read())
+            if step_df.shape[0] == 0:
+                return None
+            # encode as unicode
+            # https://stackoverflow.com/a/63028569/13113166
+            for col, dtype in step_df.dtypes.items():
+                if dtype == np.object:  # Only process byte object columns.
+                    step_df[col] = step_df[col].apply(lambda x: x.decode("utf-8"))
 
-            step_df = step_df.rename({0: 'step_n',
-                                      1: 'timestamp',
-                                      2: 'name'}, axis='columns')
+            # filter to step only
+            step_df = step_df[step_df['type'] == 'step'].drop('type', axis=1)
+            # rename and retype
+            step_df = step_df.rename(columns={
+                'value': 'step_n',
+                'time': 'timestamp',
+                'name': 'name'})
 
             step_df['timestamp'] = pd.to_datetime(step_df['timestamp'],
                                                   format='%y%m%d-%H%M%S')
+            step_df['step_n'] = pd.to_numeric(step_df['step_n'])
+
 
         else:
             group_name = "/data/{}".format(self.protocol_name)
@@ -1101,7 +1161,7 @@ N Sessions: {}""".format(self.name, path, df.shape[0], len(df.session.unique()))
                     ts = step_tab.read(start=0, stop=1, field=ts_column)
 
                 except IndexError:
-                    Warning('No Timestamp column found, only returning step numbers and named that were reached')
+                    self.logger.warning('No Timestamp column found, only returning step numbers and named that were reached')
                     ts = 0
 
                 step_df = pd.DataFrame(
@@ -1232,7 +1292,7 @@ N Sessions: {}""".format(self.name, path, df.shape[0], len(df.session.unique()))
             # TODO: Make this more robust - don't assume we got a start weight
             h5f.root.history.weights.cols.stop[-1] = stop
         else:
-            Warning("Need either a start or a stop weight")
+            self.logger.warning("Need either a start or a stop weight")
 
         _ = self.close_hdf(h5f)
 
@@ -1241,7 +1301,7 @@ N Sessions: {}""".format(self.name, path, df.shape[0], len(df.session.unique()))
         Increase the current step by one, unless it is the last step.
         """
         if len(self.current)<=self.step+1:
-            Warning('Tried to graduate from the last step!\n Task has {} steps and we are on {}'.format(len(self.current), self.step+1))
+            self.logger.warning('Tried to graduate from the last step!\n Task has {} steps and we are on {}'.format(len(self.current), self.step+1))
             return
 
         # increment step, update_history should handle the rest
